unit MainFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ActnPopupCtrl, XPStyleActnCtrls, ActnList, ActnMan,
  ComCtrls, XPMan, StdCtrls, StrUtils, TrayIcon, ImgList;

const
  AllowChar: string = '1234567890.';

type
  TMainForm = class(TForm)
    XPManifest1: TXPManifest;
    ActionManager1: TActionManager;
    ActionPower1: TAction;
    ActionTune1: TAction;
    ActionSetting1: TAction;
    ActionExit1: TAction;
    PopupActionBarEx1: TPopupActionBarEx;
    PopupPower1: TMenuItem;
    PopupTune1: TMenuItem;
    PopupSetting1: TMenuItem;
    PopupExit1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    ListView1: TListView;
    UpDown1: TUpDown;
    EditFreq1: TEdit;
    EditName1: TEdit;
    ButtonAdd1: TButton;
    TrayIcon1: TTrayIcon;
    ButtonClose1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure ButtonClose1Click(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure EditFreq1KeyPress(Sender: TObject; var Key: Char);
    procedure EditFreq1Change(Sender: TObject);
    procedure ButtonAdd1Click(Sender: TObject);
    procedure ListView1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionTune1Execute(Sender: TObject);
    procedure ActionPower1Execute(Sender: TObject);
    procedure ActionSetting1Execute(Sender: TObject);
    procedure ActionExit1Execute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private 錾 }
    ListFile: string;
    function IntToFreq(Value: Integer): string;
    function FreqToInt(S: string; Def: Integer): Integer;
    procedure SetStationMenu;
    procedure OnTune(Sender: TObject);
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

function RadioOpen: Integer; stdcall; external 'radio.dll' name 'open_radio';

function RadioClose: Integer; stdcall; external 'radio.dll' name 'close_radio';

function RadioGetStereo: Integer; stdcall; external 'radio.dll' name 'get_stereo';

function RadioSetFreq(Ch: Integer; Japan: Integer): Integer; stdcall; external 'radio.dll' name 'set_freq';

function RadioTurnPower(Flag: Integer): Integer; stdcall; external 'radio.dll' name 'turn_power';

function TMainForm.IntToFreq(Value: Integer): string;
begin
  Result := StuffString(IntToStr(Value), 3, 0, '.');
end;

function TMainForm.FreqToInt(S: string; Def: Integer): Integer;
begin
  Result := StrToIntDef(StringReplace(S, '.', '', [rfReplaceAll]), Def);
end;

procedure TMainForm.SetStationMenu;
var
  Cnt: Integer;
  MenuItem: TMenuItem;
begin
  // Set station list to menu
  PopupTune1.Clear;
  for Cnt := 0 to ListView1.Items.Count - 1 do
  begin
    MenuItem := TMenuItem.Create(PopupActionBarEx1);
    MenuItem.Tag := FreqToInt(ListView1.Items[Cnt].Caption, 830);
    MenuItem.Caption := ListView1.Items[Cnt].SubItems[0];
    MenuItem.OnClick := OnTune;
    MenuItem.ImageIndex := 2;
    PopupTune1.Add(MenuItem);
  end;
end;

procedure TMainForm.OnTune(Sender: TObject);
begin
  if not ActionPower1.Checked then
  begin
    ActionPower1.Checked := True;
    ActionPower1Execute(ActionPower1);
  end;
  if (RadioOpen = 0) then
    Exit;
  Sleep(500);
  RadioSetFreq(Trunc(TMenuItem(Sender).Tag * 20 / 10), 1);
  RadioClose;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Cnt: Integer;
  ListItem: TListItem;
begin
  ListFile := ExtractFilePath(Application.ExeName) + 'list.txt';
  // Load station list
  if FileExists(ListFile) then
  begin
    with TStringList.Create do
    try
      LoadFromFile(ListFile);
      for Cnt := 0 to Count div 2 - 1 do
      begin
        ListItem := ListView1.Items.Add;
        ListItem.Caption := Strings[Cnt * 2];
        ListItem.SubItems.Add(Strings[Cnt * 2 + 1]);
      end;
    finally
      Free;
    end;
  end;
  // Set station list to menu
  SetStationMenu;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if ActionPower1.Checked then
  begin
    ActionPower1.Checked := False;
    ActionPower1Execute(ActionPower1);
  end;
end;

procedure TMainForm.FormHide(Sender: TObject);
var
  Cnt: Integer;
begin
  // Save station list
  with TStringList.Create do
  try
    for Cnt := 0 to ListView1.Items.Count - 1 do
    begin
      Add(ListView1.Items[Cnt].Caption);
      Add(ListView1.Items[Cnt].SubItems[0]);
    end;
    SaveToFile(ListFile);
  finally
    Free;
  end;
  // Set station list to menu
  SetStationMenu;
end;

procedure TMainForm.ButtonClose1Click(Sender: TObject);
begin
  MainForm.Visible := False;
end;

procedure TMainForm.UpDown1Click(Sender: TObject; Button: TUDBtnType);
begin
  // Set position to edit
  EditFreq1.Text := IntToFreq(UpDown1.Position)
end;

procedure TMainForm.EditFreq1KeyPress(Sender: TObject; var Key: Char);
begin
  // Limit character
  if (Pos(Key, AllowChar) = 0) and (Byte(Key) <> VK_BACK) then
    Key := #0;
end;

procedure TMainForm.EditFreq1Change(Sender: TObject);
begin
  // Set position on edit change
  UpDown1.Position := FreqToInt(EditFreq1.Text, UpDown1.Position);
end;

procedure TMainForm.ButtonAdd1Click(Sender: TObject);
var
  ListItem: TListItem;
  Int: Integer;
begin
  // Add station
  if EditFreq1.Text = '' then
    MessageBox(MainForm.Handle, 'g͂Ă܂B', 'G[', MB_ICONERROR)
  else if (EditName1.Text = '') then
    MessageBox(MainForm.Handle, 'ǖ͂Ă܂B', 'G[', MB_ICONERROR)
  else
  begin
    Int := FreqToInt(EditFreq1.Text, UpDown1.Position);
    ListItem := ListView1.Items.Add;
    ListItem.Caption := IntToFreq(Int);
    ListItem.SubItems.Add(EditName1.Text);
  end;
end;

procedure TMainForm.ListView1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 46 then
    if ListView1.Items.Count > 0 then
      if ListView1.ItemFocused.Index >= 0 then
        ListView1.Items.Delete(ListView1.ItemFocused.Index);
end;

procedure TMainForm.ActionPower1Execute(Sender: TObject);
begin
  if (RadioOpen = 0) then
    Exit;
  if ActionPower1.Checked then
    RadioTurnPower(1)
  else
    RadioTurnPower(0);
  RadioClose;
end;

procedure TMainForm.ActionTune1Execute(Sender: TObject);
begin
  //
end;

procedure TMainForm.ActionSetting1Execute(Sender: TObject);
begin
  MainForm.Show;
end;

procedure TMainForm.ActionExit1Execute(Sender: TObject);
begin
  MainForm.Close;
end;

end.

